IF NOT EXISTS 
( 
    SELECT * FROM sys.objects WHERE object_id = object_id ( N'[dbo].[Proc4HistoryTableCleanup]' ) 
) 
BEGIN 
    EXECUTE 
    ( 'CREATE PROCEDURE [dbo].[Proc4HistoryTableCleanup] ( ' + 
        '    @TableOwner varchar(64),' + 
        '    @TableName     varchar(512), ' + 
        '    @DateFieldName varchar(50), ' + 
        '  @retentionDays INT = 0,' + 
        '    @Debug      TINYINT = 0' + 
        ') ' + 
        'AS ' + 
        'BEGIN ' + 
        '   SELECT ''Not implemented'' ' + 
        'END' ) 
END 
GO 

ALTER PROCEDURE [dbo].[Proc4HistoryTableCleanup] 
( 
    @TableOwner varchar ( 64 ) , 
    @TableName varchar ( 64 ) , 
    @DateFieldName varchar ( 64 ) , 
    @retentionDays int , 
    @maxRowCnt int = 10000 , 
    @debug tinyint = 0 
) 
AS 
/*
===================================================================================
DESCRIPTION:
This procedure takes in charge the clean-up for a given table based on a date field
given as parameter and a retention expressed as a number of days.
If the number of records to delete is more than a configurable limit, a dichotomic 
delete will be performed so that there is no matter on big open transaction without 
commit.
 
ARGUMENTS :
@TableOwner  name of the owner of the table to clean up
@TableName      name of the table to clean up
@DateFieldName name of the date field to use for clean up
@retentionDays number of days to keep track
@maxRowCnt  maximal number of rows to consider in an open transaction.
If the cleanup needs more than that number, the query will be cut 
into pieces implying maximum @maxRowCnt
@debug   if set to 1, this enables the debug mode
 
REQUIREMENTS:
 
==================================================================================
BUGS:
 
BUGID       Fixed   Description
==========  =====   ========================================================== 
----------------------------------------------------------------------------------
==================================================================================
NOTES:
AUTHORS:
.   VBO     Vincent Bouquette   (vincent.bouquette@chu.ulg.ac.be)
.   BBO     Bernard Bozert      (bernard.bozet@chu.ulg.ac.be)
.   JEL     Jefferson Elias     (jelias@chu.ulg.ac.be)
 
COMPANY: CHU Liege
==================================================================================
Revision History
 
Date        Nom         Description
==========  =====       ==========================================================
18/11/2014  JEL         Version 0.1.0
----------------------------------------------------------------------------------
===================================================================================
*/ 

BEGIN 

    --SET NOCOUNT ON;

    DECLARE @versionNb varchar ( 16 ) = '0.1.0' ; 
    DECLARE @tsql nvarchar ( max ) ; -- text to execute via dynamic SQL
    DECLARE @tmpRetentionDays INT; -- number of retention days to remove 
    DECLARE @retentionInc INT; -- increment for tmpRetentionDays

    BEGIN 
        TRY 
        -- 
        -- check parameter :
        --   retention days 
        --
        IF @retentionDays < 0 
        BEGIN 
            RAISERROR ( 'Parameter "retentionDays" is negative. Avorting execution' , 10 , 1 , @retentionDays ) ; 
        END 

        IF @retentionDays = 0 
        RETURN 

        --
        -- check parameters
        --   table-related parameters are usable ?
        --
        IF ( 
            NOT EXISTS 
            ( 
                SELECT 1 FROM information_schema.COLUMNS 
                WHERE TABLE_CATALOG = db_name ( db_id ( ) ) -- in current database
                AND TABLE_SCHEMA = @TableOwner 
                AND TABLE_NAME = @TableName 
                AND COLUMN_NAME = @DateFieldName 
            ) 
        ) 

        BEGIN 
            RAISERROR ( 'Parameters about the table are unusable in current database ' , 10 , 1 , @TableOwner , @TableName , @DateFieldName ) 
        END 

        IF @debug = 1 
        BEGIN 
            PRINT '----------------------------------------------------' 
            PRINT object_name ( @@PROCID ) 
            PRINT '====================================================' 
            PRINT 'Owner   = ' + @TableOwner 
            PRINT 'TableName  = ' + @TableName 
            PRINT 'Column     = ' + @DateFieldName 
            PRINT 'Retention  = ' + convert ( varchar , @retentionDays ) 
            PRINT 'Max Row Nb = ' + convert ( varchar , @maxRowCnt ) 
            PRINT '----------------------------------------------------' 
            PRINT char ( 10 ) 
        END 

        -- 
        -- Iteration number evaluation
        --
        DECLARE @dateThresh datetime 
        SET @dateThresh = getdate ( ) - @retentionDays 

        DECLARE @dateThreshStr varchar ( 100 ) 
        SELECT @dateThreshStr = convert ( varchar ( 100 ) , @dateThresh , 112 ) 

        DECLARE @totalRecordNb bigint 
        DECLARE @MinDateFieldVal datetime 

        SET @tsql = N'SELECT ' + char ( 10 ) + 
        '  @totalRecNb = COUNT_BIG(*),' + char ( 10 ) + 
        '    @minDate    = MIN([' + @DateFieldName + '])' + char ( 10 ) + 
        'FROM' + char ( 10 ) + 
        '    [' + @TableOwner + '].[' + @TableName + ']' + char ( 10 ) + 
        'WHERE' + char ( 10 ) + 
        '    ' + @DateFieldName + ' < convert(DATETIME,@dateThresh,112)' 

        EXECUTE sp_executesql @tsql , N'@totalRecNb BIGINT OUTPUT, @minDate DATETIME OUTPUT,@dateThresh VARCHAR(100)' , @minDate = @MinDateFieldVal OUTPUT , @totalRecNb = @totalRecordNb OUTPUT , @dateThresh = @dateThreshStr 

        IF @debug = 1 
        BEGIN 
            PRINT 'Min Date in table : ' + convert ( varchar , @MinDateFieldVal , 112 ) 
            PRINT 'Number of records : ' + convert ( varchar , @totalRecordNb ) 
        END 

        IF ( @totalRecordNb > @maxRowCnt ) 
        BEGIN 

            -- divide by 2 the number of days between min date and the retention period
            DECLARE @nbOfDaysToAdd int 
            SELECT @nbOfDaysToAdd = ( datediff ( day , @MinDateFieldVal , @dateThresh ) / 2 ) 

            IF @debug = 1 
            BEGIN 
                PRINT 'Too many records to take care at once !' 
                PRINT 'Days to play with : ' + convert ( varchar , @nbOfDaysToAdd ) 
            END 

            -- TODO : if days is not enough => play with hours !  

            SET @tmpRetentionDays = @retentionDays + @nbOfDaysToAdd 
            IF @debug = 1 
            BEGIN 
                PRINT 'New retention : ' + convert ( varchar , @tmpRetentionDays ) 
            END 

            EXEC [dbo].[Proc4HistoryTableCleanup] @TableOwner = @TableOwner , @TableName = @TableName , @DateFieldName = @DateFieldName , @retentionDays = @tmpRetentionDays , @maxRowCnt = @maxRowCnt , @debug = @debug 

            IF @debug = 1 
            BEGIN 
                PRINT 'Restarting procedure with the previous retention :' + convert ( varchar , @retentionDays ) 
            END 

            EXECUTE [dbo].[Proc4HistoryTableCleanup] @TableOwner = @TableOwner , @TableName = @TableName , @DateFieldName = @DateFieldName , @retentionDays = @retentionDays , @maxRowCnt = @maxRowCnt , @debug = @debug 
        END 
        ELSE 
        BEGIN 
            IF @debug = 1 
            BEGIN 
                PRINT 'Deleting ' + convert ( varchar , @totalRecordNb ) + ' records' 
            END 

            SET @tsql = 'DELETE' + char ( 10 ) + 
            'FROM' + char ( 10 ) + 
            '    [' + @TableOwner + '].[' + @TableName + ']' + char ( 10 ) + 
            'WHERE' + char ( 10 ) + 
            '    ' + @DateFieldName + ' < convert(DATETIME,@dateThresh,112)' 

            EXECUTE sp_executesql @tsql , N'@dateThresh VARCHAR(100)' , @dateThresh = @dateThreshStr 

            IF @debug = 1 
            BEGIN 
                SET @tsql = N'SELECT ' + char ( 10 ) + 
                '  @totalRecNb = COUNT_BIG(*),' + char ( 10 ) + 
                '    @minDate    = MIN([' + @DateFieldName + '])' + char ( 10 ) + 
                'FROM' + char ( 10 ) + 
                '    [' + @TableOwner + '].[' + @TableName + ']' + char ( 10 ) + 
                'WHERE' + char ( 10 ) + 
                '    ' + @DateFieldName + ' < convert(DATETIME,@dateThresh,112)' 

                EXECUTE sp_executesql @tsql , N'@totalRecNb BIGINT OUTPUT, @minDate DATETIME OUTPUT,@dateThresh VARCHAR(100)' , @minDate = @MinDateFieldVal OUTPUT , @totalRecNb = @totalRecordNb OUTPUT , @dateThresh = @dateThreshStr 

                PRINT 'Nb of records after delete : ' + convert ( varchar , @totalRecordNb ) 
                PRINT 'Minimum date after delete  : ' + convert ( varchar , @MinDateFieldVal , 112 ) 
            END 

        END 

    END 
    TRY 

    BEGIN 
        CATCH 
        PRINT 'ErrorNumber    : ' + convert ( varchar , ERROR_NUMBER ( ) ) 
        PRINT 'ErrorSeverity  : ' + convert ( varchar , ERROR_SEVERITY ( ) ) 
        PRINT 'ErrorState     : ' + convert ( varchar , ERROR_STATE ( ) ) 
        PRINT 'ErrorProcedure : ' + convert ( varchar , ERROR_PROCEDURE ( ) ) 
        PRINT 'ErrorLine      : ' + convert ( varchar , ERROR_LINE ( ) ) 
        PRINT 'ErrorMessage   : ' + convert ( varchar , ERROR_MESSAGE ( ) ) 
        IF @@TRANCOUNT > 0 
        ROLLBACK TRAN --RollBack in case of Error

        RAISERROR ( 'Unable to proceed !' , 10 , 1 , @retentionDays ) ; 

    END 
    CATCH 
END 

/**

Usage example :
==============
exec dbo.Proc4HistoryTableCleanup 
@TableOwner  = 'dbo',
@TableName  = 'CPUStatsHistory',
@DateFieldName = 'DateStamp',
@retentionDays  = 700,
@debug   = 1

exec dbo.Proc4HistoryTableCleanup 
@TableOwner  = 'dbo',
@TableName  = 'CPUStatsHistory',
@DateFieldName = 'DateStamp',
@retentionDays  = 300,
@debug   = 1  
*/ 

ALTER TABLE dbo.DataDictionary_Tables 
ADD retentionDays int DEFAULT 90 

UPDATE dbo.DataDictionary_Tables 
SET retentionDays = NULL 
WHERE TableName IN 
( 
    'AlertContacts' , 'AlertSettings' , 'DatabaseSettings' , 'DataDictionary_Fields' , 
    'DataDictionary_Tables' , 'SchemaChangeLog' , 'ServerChangeLog' 
) 

UPDATE dbo.DataDictionary_Tables 
SET retentionDays = 90 
WHERE TableName IN 
( 
    'BlockingHistory' , 'CPUStatsHistory' , 'FileStatsHistory' , 'HealthReport' , 'JobStatsHistory' , 
    'MemoryUsageHistory' , 'PerfStatsHistory' , 'QueryHistory' 
) 

ALTER TABLE dbo.DataDictionary_Tables 
ADD dateField varchar ( 100 ) 

UPDATE dbo.DataDictionary_Tables 
SET dateField = 'DateStamp' 
WHERE TableName IN 
( 
    'BlockingHistory' , 'CPUStatsHistory' , 'HealthReport' , 
    'MemoryUsageHistory' , 'QueryHistory' 
) 
UPDATE dbo.DataDictionary_Tables 
SET dateField = 'FileStatsDateStamp' 
WHERE TableName = 'FileStatsHistory' 

UPDATE dbo.DataDictionary_Tables 
SET dateField = 'JobStatsDateStamp' 
WHERE TableName = 'JobStatsHistory' 

UPDATE dbo.DataDictionary_Tables 
SET dateField = 'StatDate' 
WHERE TableName = 'PerfStatsHistory' 

CREATE VIEW dbo.CleanupSettings 
AS 
SELECT 
SchemaName , 
TableName , 
dateField AS DateFieldName , 
retentionDays 
FROM dbo.DataDictionary_Tables WHERE retentionDays IS NOT null; 

IF NOT EXISTS 
( 
    SELECT * FROM sys.objects WHERE object_id = object_id ( N'[dbo].[dbWarden_HistoryCleanup]' ) 
) 
BEGIN 
    EXECUTE 
    ( 'CREATE PROCEDURE [dbo].[dbWarden_HistoryCleanup] ( ' + 
        '    @Debug      TINYINT = 0' + 
        ') ' + 
        'AS ' + 
        'BEGIN ' + 
        '   SELECT ''Not implemented'' ' + 
        'END' ) 
END 
GO 

ALTER PROCEDURE [dbo] .dbWarden_HistoryCleanup 
( 
    @debug tinyint = 0 
) 
AS 
/*
===================================================================================
DESCRIPTION:
This procedure takes care of purging old data inside tables defined in the
table or view dbo.CleanupSettings of the same database as the one in which this
procedure has been created.
 
ARGUMENTS :
@debug   if set to 1, this enables the debug mode
 
REQUIREMENTS:
 
==================================================================================
BUGS:
 
BUGID       Fixed   Description
==========  =====   ========================================================== 
----------------------------------------------------------------------------------
==================================================================================
NOTES:
AUTHORS:
.   VBO     Vincent Bouquette   (vincent.bouquette@chu.ulg.ac.be)
.   BBO     Bernard Bozert      (bernard.bozet@chu.ulg.ac.be)
.   JEL     Jefferson Elias     (jelias@chu.ulg.ac.be)
 
COMPANY: CHU Liege
==================================================================================
Revision History
 
Date        Nom         Description
==========  =====       ==========================================================
19/11/2014  JEL         Version 0.1.0
----------------------------------------------------------------------------------
===================================================================================
*/ 

BEGIN 

    --SET NOCOUNT ON;

    DECLARE @versionNb varchar ( 16 ) = '0.1.0' ; 
    DECLARE @tsql nvarchar ( max ) ; -- text to execute via dynamic SQL

    DECLARE @CurrentOwner varchar ( 50 ) 
    DECLARE @CurrentTable varchar ( 50 ) 
    DECLARE @CurrentColumn varchar ( 50 ) 
    DECLARE @CurrentRetention int 

    DECLARE getTablesToPurge CURSOR FOR 
    SELECT * FROM [dbo].[CleanupSettings] 

    OPEN getTablesToPurge 
    FETCH NEXT 
    FROM getTablesToPurge INTO @CurrentOwner , @CurrentTable , @CurrentColumn , @CurrentRetention 

    WHILE @@FETCH_STATUS = 0 
    BEGIN 

        IF @debug = 1 
        BEGIN 
            PRINT 'Current Owner  : ' + @CurrentOwner 
            PRINT 'Current Table  : ' + @CurrentTable 
            PRINT 'Current Column : ' + @CurrentColumn + '(datetime column used in where clause)' 
            PRINT 'Current retention : ' + convert ( varchar , @CurrentRetention ) 
        END 

        EXEC dbo.Proc4HistoryTableCleanup 
        @TableOwner = @CurrentOwner , 
        @TableName = @CurrentTable , 
        @DateFieldName = @CurrentColumn , 
        @retentionDays = @CurrentRetention , 
        @debug = @debug 

        -- carry on ...
        FETCH NEXT 
        FROM getTablesToPurge INTO @CurrentOwner , @CurrentTable , @CurrentColumn , @CurrentRetention 
    END 
    CLOSE getTablesToPurge 
    DEALLOCATE getTablesToPurge 

END 

/**

Usage example :
==============
exec dbo.[dbWarden_HistoryCleanup] 
@debug   = 1

exec dbo.[dbWarden_HistoryCleanup]
*/ 

-- 
-- ------------------------------
-- Sample job : everyday @8:15PM
-- ------------------------------
--
IF NOT EXISTS 
( 
    SELECT * FROM msdb..sysjobs WHERE name = 'dbWarden_HistoryCleanup' 
) 
BEGIN 
    BEGIN TRANSACTION 
    DECLARE @ReturnCode int 
    SELECT @ReturnCode = 0 

    DECLARE @jobId binary ( 16 ) 

    EXEC 
    @ReturnCode = msdb..sp_add_job @job_name = N'dbWarden_HistoryCleanup' , 
    @enabled = 1 , 
    @notify_level_eventlog = 0 , 
    @notify_level_email = 2 , 
    @notify_level_netsend = 0 , 
    @notify_level_page = 0 , 
    @delete_level = 0 , 
    @description = N'No description available.' , 
    @category_name = N'Database Monitoring' , 
    @owner_login_name = N'sa' , 
    @notify_email_operator_name = N'SQL_DBA' , @job_id = @jobId OUTPUT 

    IF ( @@ERROR <> 0 OR @ReturnCode <> 0 ) 
    GOTO QuitWithRollback 

    EXEC 
    @ReturnCode = msdb..sp_add_jobstep @job_id = @jobId , @step_name = N'run proc' , 
    @step_id = 1 , 
    @cmdexec_success_code = 0 , 
    @on_success_action = 1 , 
    @on_success_step_id = 0 , 
    @on_fail_action = 2 , 
    @on_fail_step_id = 0 , 
    @retry_attempts = 0 , 
    @retry_interval = 0 , 
    @os_run_priority = 0 , @subsystem = N'TSQL' , 
    @command = N'SET NOCOUNT ON
    EXEC [DBA].dbo.dbWarden_HistoryCleanup' , 
    @database_name = N'master' , 
    @flags = 0 

    IF ( @@ERROR <> 0 OR @ReturnCode <> 0 ) 
    GOTO QuitWithRollback 

    EXEC @ReturnCode = msdb..sp_update_job @job_id = @jobId , @start_step_id = 1 

    DECLARE @schedule_id int 

    EXEC msdb.dbo.sp_add_jobschedule @job_id = @jobId , @name = N'dbWarden_Schedule_HistoryCleanup' , 
    @enabled = 1 , 
    @freq_type = 4 , 
    @freq_interval = 1 , 
    @freq_subday_type = 1 , 
    @freq_subday_interval = 0 , 
    @freq_relative_interval = 0 , 
    @freq_recurrence_factor = 1 , 
    @active_start_date = 20141119 , 
    @active_end_date = 99991231 , 
    @active_start_time = 201500 , 
    @active_end_time = 235959 , @schedule_id = @schedule_id OUTPUT 

    IF ( @@ERROR <> 0 OR @ReturnCode <> 0 ) 
    GOTO QuitWithRollback 

    EXEC @ReturnCode = msdb..sp_add_jobserver @job_id = @jobId , @server_name = N'(local)' 
    IF ( @@ERROR <> 0 OR @ReturnCode <> 0 ) 
    GOTO QuitWithRollback 

    COMMIT TRANSACTION 
    GOTO EndSave 
    QuitWithRollback: 
    IF ( @@TRANCOUNT > 0 ) ROLLBACK TRANSACTION 
    EndSave: 
END 
GO